/****************************************************************************************/
/*																						*/
/*						time_tester - ACK timer test task.								*/
/*																						*/
/*																						*/
/*					Richard W. Mincher.						05/01/87.					*/
/*																						*/
/*				Copyright  1987-88, Apple Computer, Inc.  All rights reserved.			*/
/*																						*/
/****************************************************************************************/

#include "os.h"
#include "managers.h"

pascal	void	illegal ()
		extern	0x4afc;

static	message	*msg_ptr;

time_tester ()
{
	message	*msg;
	long	mid;
	short	goaway;
	void	delay ();

#ifdef	PRINT	
	printf("TID = %x - Time tester task starting.\n", GetTID ());
#endif	PRINT

	while(GetTimerTID() == 0)
		delay (3);
		
	/*	send a 1 shot request	*/

	if ((msg_ptr = GetMsg()) == 0)			/*	Get message buffer	*/
		illegal();
		
	msg_ptr->mCode = TIMER_1_SHOT_REQUEST;
	msg_ptr->mOData[0] = 300;
	msg_ptr->mTo = GetTimerTID();
	mid = msg_ptr->mId;
	Send( msg_ptr );
	msg_ptr = 0;
	
#ifdef	PRINT
	printf("TID = %x - Waiting for 1-shot\n", GetTID ());
#endif	PRINT

	msg = Receive( mid, OS_MATCH_ALL, OS_MATCH_ALL, OS_NO_TIMEOUT );
	
#ifdef	PRINT
	if (msg->mId == mid)
		printf("TID = %x - 1-shot received\n", GetTID ());
	else
		printf("TID = %x - 1-shot not received.\n", GetTID ());
#endif	PRINT

	if (msg)
		FreeMsg( msg );
	else
		illegal();
	
	/*	Send a periodic and a 1-shot	*/
	
	if ((msg_ptr = GetMsg()) == 0)			/*	Get message buffer	*/
		illegal();
		
	msg_ptr->mCode = TIMER_1_SHOT_REQUEST;
	msg_ptr->mOData[0] = 5000;
	msg_ptr->mTo = GetTimerTID();
	Send( msg_ptr );	
	msg_ptr = 0;
	
	if ((msg = GetMsg()) == 0)			/*	Get message buffer	*/
		illegal();
		
	msg->mCode = TIMER_PERIODIC_REQUEST;
	msg->mOData[0] = 300;
	msg->mOData[1] = 1000;
	msg->mTo = GetTimerTID();
	mid = msg->mId;
	Send( msg );
	msg = 0;
	
	goaway = 0;
	
	while(!goaway && (msg = Receive( OS_MATCH_ALL, OS_MATCH_ALL, OS_MATCH_ALL, OS_NO_TIMEOUT )))
	{
		switch( msg->mCode )
		{
			case	TIMER_1_SHOT_REPLY:
			
#ifdef	PRINT
				printf("TID = %x - 1-shot reply received.\n", GetTID ());
#endif	PRINT

				if ((msg_ptr = GetMsg()) == 0)
				{
					illegal();
					illegal();
					illegal();
				}
				
				msg_ptr->mTo = GetTimerTID();
				msg_ptr->mOData[0] = mid;
				msg_ptr->mCode = TIMER_CANCEL_REQUEST;
				Send( msg_ptr );
				break;
				
			case	TIMER_PERIODIC_REPLY:
			
#ifdef	PRINT
				printf("TID = %x - Periodic at %d\n", GetTID (), GetETick() );
#endif	PRINT

				break;
				
			case	TIMER_CANCEL_REPLY:
			
#ifdef	PRINT
				printf("TID = %x - Timer cancel reply, count = %d\n", GetTID (),
					msg->mOData[1] );
#endif	PRINT

				break;
			
			default:
#ifdef	PRINT
				printf("TID = %x - Bad message returned.  ptr=%x, mFrom = %x, mCode = %x\n", 
					GetTID (), msg, msg->mFrom, msg->mCode);
#endif	PRINT
				break;
		}
		
		if (msg != 0)
			FreeMsg( msg );
		else
		{
			illegal();
			illegal();
			illegal();
			illegal();
			illegal();
		}
	}
	
	/*	Wrap up	*/
	
#ifdef	PRINT
	printf("TID = %x - Time Tester task going away.\n", GetTID ());	
#endif	PRINT
}


static void delay (timeout)
long timeout;
{
	/* Delay is done by waiting for a message that will never arrive! */
	
	(void) Receive (OS_MATCH_NONE, OS_MATCH_NONE, OS_MATCH_NONE, timeout);	
}