/************************************************************************/
/*																		*/
/*		trace_manager - A/ROSE Trace Manager								*/
/*																		*/
/*		Richard K. Nordin			11/20/87.							*/
/*																		*/
/*		Copyright  1987-88, Apple Computer, Inc.  All rights reserved.	*/
/*																		*/
/************************************************************************/


#include <Types.h>
#include "os.h"
#include "arose.h"
#include "managers.h"

#define	TRUE		1
#define	FALSE		0

#define	ERRORCODE(xx)		((xx) |= 0x8000)
#define PREVERROR(xx)		((xx) &  0x8000)

pascal void
illegal()
	extern	0x4afc;
		
		
Boolean			Tracing;					/* Tracing Flag */
tid_type		Monitor_tid;				/* TID of Trace Monitor */
tid_type		My_tid;						/* TID of Trace Manager */


trace_manager()
{
	message		*msg;								/* Pointer to Message */
	static char	my_object_name[] = "Trace Manager";	/* Trace Manager Object Name */
	static char	my_type_name[] = "Trace Manager";	/* Trace Manager Type Name */
	tid_type	lookup_monitor();					/* Lookup Trace Monitor */
	void		process_forwarded_message();		/* Process Message Forwarded by Send */
	void		process_my_message();				/* Process Message Directed to Trace Manager */
	
	/* Initialize. */

#ifdef PRINT
	printf("TID %x: Trace Manager: Starting.\n", GetTID());
#endif PRINT
		
	My_tid = GetTID();

	while (0 == Register_Task(my_object_name, my_type_name, Machine_Visible))
	{
		(void) Receive(OS_MATCH_NONE, OS_MATCH_NONE, OS_MATCH_NONE, 4);	/* Wait. */
	}
	
	/* Wait for Trace Monitor. */
	
	while (0 == (Monitor_tid = lookup_monitor()))
	{
		(void) Receive(OS_MATCH_NONE, OS_MATCH_NONE, OS_MATCH_NONE, GetTickPS());		/* Wait. */
	}
	
	Tracing = TRUE;
	
	/* Register as Trace Manager. */
	
	TraceReg();
	
	/* Process messages. */

	while (TRUE)
	{
		msg = Receive(OS_MATCH_ALL, OS_MATCH_ALL, OS_MATCH_ALL, OS_NO_TIMEOUT);
		if (msg == NULL)
		{
			illegal();
		}
		
		if (msg->mTo == My_tid)
		{
			process_my_message(msg);
		}
		else
		{
			process_forwarded_message(msg);
		}
	}
}


/*	process_my_message - Process Message Directed to Trace Manager
 *
 */
 

void
process_my_message(msg)
message		*msg;		/* Pointer to Message */
{	
	switch (msg->mCode)
	{
	case TM_TRACE_ON:
		Tracing = TRUE;
		Monitor_tid = msg->mFrom;
		FreeMsg(msg);
		break;
		
	case TM_TRACE_OFF:
		Tracing = FALSE;
		Monitor_tid = msg->mFrom;
		FreeMsg(msg);
		break;
		
	default:				
		if (PREVERROR(msg->mCode))
		{
			FreeMsg(msg);
		}
		else
		{
			ERRORCODE(msg->mCode);
			
			msg->mStatus = TME_UNKNOWN_MESSAGE;
			msg->mTo = msg->mFrom;
			msg->mFrom = My_tid;
			Send(msg);
		}
		break;
	}
}


/*	process_forwarded_message - Process Message Forwarded by Send
 *
 */
 

void
process_forwarded_message(msg)
message		*msg;		/* Pointer to Message */
{
	char		*copy_msg;		/* Pointer to Copy of Message/Data */
	message		*monitor_msg;	/* Pointer to Trace Monitor Message */
	
	if (Tracing)
	{
		/* Copy message. */
		
		while (NULL == (copy_msg = GetMem(sizeof (message) + msg->mDataSize)))
		{
			(void) Receive(OS_MATCH_NONE, OS_MATCH_NONE, OS_MATCH_NONE, 2);	/* Wait. */
		}
		
		BlockMove(msg, copy_msg, sizeof (message));
		NetCopy(msg->mFrom, msg->mDataPtr, GetTID(), copy_msg + sizeof (message), msg->mDataSize);
		
		/* Send to Trace Monitor. */

		while (NULL == (monitor_msg = GetMsg()))
		{
			(void) Receive(OS_MATCH_NONE, OS_MATCH_NONE, OS_MATCH_NONE, 2);	/* Wait. */
		}
		
		monitor_msg->mTo = Monitor_tid;
		monitor_msg->mFrom = My_tid;
		monitor_msg->mCode = TM_TRACE;
		monitor_msg->mDataPtr = copy_msg;		
		monitor_msg->mDataSize = sizeof (message) + msg->mDataSize;
		Send(monitor_msg);
		
		monitor_msg = Receive(OS_MATCH_ALL, Monitor_tid, TM_TRACE + 1, 5*GetTickPS());
		if (monitor_msg == NULL)
		{
			Tracing = FALSE;			/* Trace Monitor timed out. */
		}
		else
		{
			FreeMsg(monitor_msg);
		}
		
		FreeMem(copy_msg);
	}

	Send(msg);				/* Send message on to destination. */
	
	return;
}


/*	lookup_monitor - Lookup Trace Monitor
 *
 */
 
 
tid_type
lookup_monitor()
{
	int					i;									/* Temporary Index */
	unsigned short		index;								/* Name Lookup Index */
	message				*msg;								/* Pointer to Message */
	static char			object_name[] = "Trace Monitor";	/* Trace Monitor Object Name */
	static char			type_name[] = "Trace Monitor";		/* Trace Monitor Type Name */
	struct ra_GetCards	get_cards;							/* ICC GetCards Return Area */
	tid_type			monitor_tid;						/* TID of Trace Monitor */
	
	/* Get TIDs of Name Managers. */
	
	while (0 == (msg = GetMsg()))
	{
		(void) Receive(OS_MATCH_NONE, OS_MATCH_NONE, OS_MATCH_NONE, 3);	/* Wait. */
	}
	
	msg->mTo = GetICCTID();
	msg->mFrom = GetTID();
	msg->mCode = ICC_GETCARDS;
	msg->mDataPtr = (char *) &get_cards;
	msg->mDataSize = sizeof(get_cards);
	Send(msg);
	
	msg = Receive(OS_MATCH_ALL, OS_MATCH_ALL, ICC_GETCARDS + 1, OS_NO_TIMEOUT);	
	if (msg != NULL)
	{
		FreeMsg(msg);
	}
	
	/* Lookup Trace Monitor. */
	
	for (i = 0; i < IC_MAXCARDS; i++)
	{
		if (get_cards.tid[i] > 0)			/* If Name Manager present... */
		{
			index = 0;
			monitor_tid = Lookup_Task(object_name, type_name, get_cards.tid[i], &index);
			
			if (monitor_tid != 0)			/* If Trace Monitor present... */
			{
				return(monitor_tid);		/* Return Trace Monitor TID. */
			}
		}
	}
	
	return(0);		/* Indicate no Trace Monitor. */
}
