/************************************************************************/
/*																		*/
/*		DumpTrace - Dump Trace File Program								*/
/*																		*/
/*		Richard K. Nordin			11/20/87.							*/
/*																		*/
/*		Copyright  1987-88, Apple Computer, Inc.  All rights reserved.	*/
/*																		*/
/************************************************************************/


/*	DumpTrace - Dump Trace File
 *
 *	DumpTrace dumps a A/ROSE message trace file or files to standard output.
 *
 *	DumpTrace [-an -cn -dn -fn -in -ln -pn -sn -tn ...] [file ...]
 *
 *	file	- File Name
 *
 *	If no file name is specified, DumpTrace dumps "TraceFile".
 *
 *	Messages are dumped selectively based on values specified by option
 *	arguments.  Each option argument can be repeated with different values.
 *	If option arguments are specified, a message is dumped only if its
 *	fields match one of the values specified by each of the arguments.
 *	Values are specified as hexadecimal numbers.
 *
 *	-an		- Dump messages having To or From values of "n"
 *	-cn		- Dump messages having Code value of "n"
 *	-dn		- Dump messages having DataPtr value of "n"
 *	-fn		- Dump messages having From value of "n"
 *	-in		- Dump messages having ID value of "n"
 *	-ln		- Dump messages having DataSize value of "n"
 *	-pn		- Dump messages having Priority value of "n"
 *	-sn		- Dump messages having Status value of "n"
 *	-tn		- Dump messages having To value of "n"
 *
 *	"DumpTrace -f0d01 -f0d02 -c64 FileName1 FileName2" will dump those
 *	messages from FileName1 and FileName2 that have Code values of 100
 *	(64 hex) and that are are from tasks 0d01 (slot d, task 1) or 0d02.
 *
 *	"DumpTrace -a0d03" dumps those messages from TraceFile that are either
 *	to or from task 0d03.
 *
 *	Trace files are of a form created by the Trace Monitor program.  A trace
 *	file consists of a sequence of A/ROSE messages.  If a message has an
 *	associated data buffer (mDataSize is nonzero), the message is immediately
 *	followed by the data buffer contents of size mDataSize. 
 *
 */
 

#include <StdIO.h>
#include <Types.h>
 

#define TRACEFILE	"TraceFile"		/* Default Trace File Name */

#define TRUE		1
#define FALSE		0


Boolean	Error = FALSE;		/* Error Flag */

 
/*	main - Main Program
 *
 *
 */
	
main(argc, argv, envp)
int		argc;		/* Argument Count */
char	*argv[];	/* Arguments */
char	*envp[];	/* Environment */
{
#pragma	unused	(envp)

	extern void	dump_trace_file();	/* Dump Trace File */
	extern void	init();				/* Initialize */
	FILE		*file;				/* File Stream */
	int			file_count;			/* Filename Argument Counter */
	int			i;					/* Temporary Index */
	
	/* Initialize. */

	init(argc, argv);
	
	/* Dump trace files. */
	
	file_count = 0;
	
	for (i = 1; i < argc; i++)
	{
		if (*argv[i] != '-')
		{
			file_count++;
		
			if ((file = fopen(argv[i], "r")) != NULL)
			{
				printf("%s\n", argv[i]);
				dump_trace_file(stdout, file);
				fclose(file);
			}
			else
			{
				fprintf(stderr, "### DumpTrace: cannot read %s\n", argv[i]);
				Error = TRUE;
			}
		}
	}
	
	if (file_count == 0)
	{
		if ((file = fopen(TRACEFILE, "r")) != NULL)
		{
			dump_trace_file(stdout, file);
			fclose(file);
		}
		else
		{
			fprintf(stderr, "### DumpTrace: cannot read %s\n", TRACEFILE);
			Error = TRUE;
		}
	}
	
	/* Exit. */
	
	if (Error)
	{
		exit(2);
	}
	else
	{
		exit(0);
	}
}
