/*
	File:		AROSEQueue.cp

	Contains:	xxx put contents here xxx

	Written by:	xxx put writers here xxx

	Copyright:	 1991 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <5>	 7/17/91	gcc		Destructor should NOT close the queue.
		 <4>	 7/17/91	gcc		Change names of AROSEQueueGlue.a table entries to x_Entry 
		 							so they won't conflict with the "regular" IPCGlue routines
		 <3>	 7/17/91	gcc		Get it working	
		 <1>	 7/15/91	gcc		first checked in

	To Do:
*/

#include "AROSEQueue.h"

// modified definitions of AROSE routines so names
// won't conflict with the "regular" glue routines
extern "C" {
extern	void			CloseQueue_Entry(void);
extern	mMessage* 		GetMsg_Entry(void);
extern	void			FreeMsg_Entry(struct mMessage *);
extern	char			GetCard_Entry(void);
extern	unsigned long	GetETick_Entry(void);
extern	tid_type		GetICCTID_Entry(void);
extern	tid_type		GetNameTID_Entry(void);
extern	unsigned short	GetTickPS_Entry(void);
extern	tid_type		GetTID_Entry(void);
extern	short			IsLocal_Entry(char *);
extern	void			KillReceive_Entry();
extern	short			LockRealArea_Entry(void *, unsigned long, struct addressareas buffer[], unsigned long);
extern	tid_type		Lookup_Task_Entry(char *, char *, tid_type, unsigned short *);
extern	short			NetCopy_Entry(tid_type, void *, tid_type, void *, unsigned long);
extern	tid_type		OpenQueue_Entry(void (*)());
extern	struct mMessage *Receive_Entry(unsigned long, tid_type, unsigned short, long, ...);
extern	char			Register_Task_Entry(char *, char *, short);
extern	void			Send_Entry(struct mMessage *);
extern	void			SwapTID_Entry(struct mMessage *);
extern	short			UnlockRealArea_Entry(void *, unsigned long);
extern	char*			AROSEGetMem_Entry(long);
extern	void			AROSEFreeMem_Entry(char *);
extern	void			TraceReg_Entry(void);
extern	tid_type		GetTraceTID_Entry(void);
extern	long			CountFreeMsgs_Entry(void);
};


TAROSEQueue::TAROSEQueue()
{
	fQueueId = 0;
}


TAROSEQueue::~TAROSEQueue()
{
	if (fQueueId)
		this -> CloseQueue ();
}


void
TAROSEQueue::CloseQueue(void)
{
	GetMyQueueId();
	::CloseQueue_Entry();
	fQueueId = 0;
}


void
TAROSEQueue::FreeMsg(mMessage *msgPtr)
{
	GetMyQueueId();
	::FreeMsg_Entry(msgPtr);
}


char
TAROSEQueue::GetCard(void)
{
	GetMyQueueId();
	return (::GetCard_Entry());
}


unsigned long
TAROSEQueue::GetETick(void)
{
	GetMyQueueId();
	return (::GetETick_Entry());
}


tid_type
TAROSEQueue::GetICCTID(void)
{
	GetMyQueueId();
	return (::GetICCTID_Entry());
}


mMessage*
TAROSEQueue::GetMsg(void)
{
	GetMyQueueId();
	return (::GetMsg_Entry());
}


tid_type
TAROSEQueue::GetNameTID(void)
{
	GetMyQueueId();
	return (::GetNameTID_Entry());
}


unsigned short	
TAROSEQueue::GetTickPS(void)
{
	GetMyQueueId();
	return (::GetTickPS_Entry());
}


tid_type
TAROSEQueue::GetTID(void)
{
	GetMyQueueId();
	return (::GetTID_Entry());
}


short
TAROSEQueue::IsLocal(char *address)
{
	GetMyQueueId();
	return (::IsLocal_Entry(address));
}


void
TAROSEQueue::KillReceive(void)
{
	GetMyQueueId();
	::KillReceive_Entry();
}


short
TAROSEQueue::LockRealArea(void *virtualAddr, unsigned long length,
						struct addressareas buffer[], unsigned long count)
{
	GetMyQueueId();
	return (::LockRealArea_Entry(virtualAddr, length, buffer, count));
}


tid_type
TAROSEQueue::Lookup_Task(char *object, char *type, tid_type nm_TID,
						unsigned short *index)
{
	GetMyQueueId();
	return (::Lookup_Task_Entry(object, type, nm_TID, index));
}


short
TAROSEQueue::NetCopy(tid_type srcTID, void *srcAddress,
					tid_type dstTID, void *dstAddress,
					unsigned long byteCount)
{
	GetMyQueueId();
	return (::NetCopy_Entry(srcTID, srcAddress, dstTID, dstAddress, byteCount));
}


tid_type
TAROSEQueue::OpenQueue(void (*UserProcedure)(void))
{
	fQueueId = 0;
	GetMyQueueId();
	return (::OpenQueue_Entry(UserProcedure));
}


mMessage*
TAROSEQueue::Receive(unsigned long mID, tid_type mFrom,
					unsigned short mCode, long timeOut,
					void (*CompletionRoutine)(mMessage *))
{
	GetMyQueueId();
	return (::Receive_Entry(mID, mFrom, mCode, timeOut, CompletionRoutine));
}


char
TAROSEQueue::Register_Task(char object[], char type[], short local_only)
{
	GetMyQueueId();
	return (::Register_Task_Entry(object, type, local_only));
}


void
TAROSEQueue::Send(mMessage *msgPtr)
{
	GetMyQueueId();
	::Send_Entry(msgPtr);
}


void
TAROSEQueue::SwapTID(mMessage *msgPtr)
{
	GetMyQueueId();
	::SwapTID_Entry(msgPtr);
}


void
TAROSEQueue::UnlockRealArea(void *virtualAddr, unsigned long length)
{
	GetMyQueueId();
	::UnlockRealArea_Entry(virtualAddr, length);
}


void			TAROSEQueue::TraceReg(void)
{
	GetMyQueueId();
	::TraceReg_Entry();
}


tid_type		TAROSEQueue::GetTraceTID(void)
{
	GetMyQueueId();
	return (::GetTraceTID_Entry());
}



char*			TAROSEQueue::AROSEGetMem(long theSize)
{
	GetMyQueueId();
	return (::AROSEGetMem_Entry(theSize));
}


void			TAROSEQueue::AROSEFreeMem(char * thePtr)
{
	GetMyQueueId();
	::AROSEFreeMem_Entry(thePtr);
}


long			TAROSEQueue::CountFreeMsgs(void)
{
	GetMyQueueId();
	return (::CountFreeMsgs_Entry());
}
