
#include <types.h>
#include <ioctl.h>
#include <errno.h>
#include <StdLib.h>
#include <String.h>

#include <memory.h>

#include <os.h>					/* Mr. DOS includes */
#include <managers.h>
#include <timerlibrary.h>

static unsigned long hsStripAddressMask;

#define	TRUE				1
#define	FALSE				0


/* 
 * Entry point : 
 */
name_dump ()
{
	tid_type 			ilTid;
	AROSEmessage		*xlMsg;
	struct ra_GetCards	lCards;
	int					mlSlot;
	unsigned short		index;
	
	hsStripAddressMask = StripAddress ( (void *)~0 );
	
	my_printf("arose name dump:\n");
	
	if ((ilTid = OpenQueue (0)) == 0)
	{
		my_printf ("No A/ROSE present\n");
		return;
	}
	
	/* See how many cards have a name manager running. */
	if (( xlMsg = GetMsg()) == nil )
	{
		my_printf("Couldn't get message\n");
		CloseQueue();
		return;
	}
	xlMsg->mTo = GetICCTID();
	xlMsg->mCode = ICC_GETCARDS;
	xlMsg->mDataPtr = (char *)&lCards;
	xlMsg->mDataSize = sizeof( struct ra_GetCards );
	Send( xlMsg );
	xlMsg = Receive( OS_MATCH_ALL, OS_MATCH_ALL, OS_MATCH_ALL, 0, 0 );
	if (( long ) xlMsg < 0 )
	{
		my_printf("Bad Send to ICC, returned %d\n", (long)xlMsg );
		CloseQueue();
		return;
	}
	for ( mlSlot = 0; mlSlot < IC_MAXCARDS; mlSlot++ )
	{
		if ( lCards.tid[mlSlot] > 0 )
		{
			my_printf("************************************\n");
			my_printf("Dump of slot 0x%x\n", mlSlot);
		
			index = 0;
			while (ilTid = My_Lookup_Task ("=", "=", lCards.tid[mlSlot], &index))
			{
			}
		}
	}
	
	CloseQueue();
	
}

/*
*	name--
*		My_Lookup Task
*
*	input--
*		object	- object name to lookup
*		type	- type name to lookup
*		nm_tid	- name manager tid
*		index	- current offset in name manager tables to start lookup from
*					(initially set to zero on first call by user)
*
*	output--
*		index	- new offset in name manager tables to start lookup from
*
*		lookup_task - The task id of the named task is returned is found else zero
*
*/
tid_type My_Lookup_Task ( object, type, nm_TID, index)
char			object [];
char 			type [];
tid_type		nm_TID;
unsigned short	*index;
{

	boolean		all_ok;
	message		*msgptr;
	long		mId;
	short		i;
	tid_type	tid;
	
	struct ltid_pb
	{	
		struct	pb_lookup_tid 	ltid;
		struct	obj_rec			last;
		unsigned long			fred;
	} *buffer, ltid_buf;
	
	/* we will try to lookup the name with the name manager if it exists */
	/* we use Receive with timeout to force all other tasks to run before us */
	
	if ((msgptr = GetMsg ()) == NULL)
		return 0;
	buffer = ( struct ltid_pb * )(((unsigned long)&ltid_buf) & hsStripAddressMask);
	
	msgptr -> mCode = NM_LOOKUP_TID;
	msgptr -> mDataPtr = (char *) buffer;
	msgptr -> mDataSize = sizeof (struct ltid_pb);
	msgptr -> mTo = nm_TID;
		
	/* calculate size of object name */
			
	i = 0;
	while ( i < NM_Obj_Size_Max && object [i] )
		i++;
	
	buffer -> ltid.ltid_on.o_len = i;
	BlockMove (	object, (char *) buffer -> ltid.ltid_on.o_name, i);

	/* calculate size of type name */
	
	i = 0;
	while ( i < NM_Type_Size_Max && type [i] )
		i++;
	
	buffer -> ltid.ltid_tn.t_len = i;
	BlockMove (type, (char *) buffer -> ltid.ltid_tn.t_name, i);
			
	buffer -> ltid.ltid_ra [0].ra_on.o_len = 0;
	buffer -> ltid.ltid_index = *index;
	buffer -> ltid.ltid_RAsize = sizeof(struct ltid_pb) - (sizeof (struct pb_lookup_tid)
														- sizeof (struct ra_ltid));
	
	mId = msgptr -> mId;
	Send (msgptr);
	
	msgptr = Receive (mId, OS_MATCH_ALL, OS_MATCH_ALL, 0, 0 );
	
	if (msgptr == 0)
		return (FALSE);
	else
		all_ok = (msgptr -> mStatus == NM_NO_ERRORS);
	tid = 0;
		
	if (all_ok)
	{
		*index = buffer -> ltid.ltid_index;

		if ( buffer-> ltid.ltid_ra [0].ra_on.o_len )		/* name found */
		{
			tid = buffer-> ltid.ltid_ra [0].ra_tid;
			
			my_printf("Task with TID 0x%x, registered as:\n",tid);
			my_printf("\tname <%s>\n", buffer->ltid.ltid_ra[0].ra_on.o_name);
			my_printf("\ttype <%s>\n", buffer->ltid.ltid_ra[0].ra_tn.t_name);
		}
	}
	FreeMsg (msgptr);
	return (tid);
}

